module BaseVersion (version, copyright, disclaimer)
where

-- version number is major.minor.patchlvl; don't change the format of the
-- `versnum' line as it is `grep'ed for by a Makefile
--
idstr      = "$Id: BaseVersion.hs,v 1.1.1.1 2004/11/13 16:42:44 duncan_coutts Exp $"
name       = "Compiler Toolkit"
versnum    = "0.26.0"
date       = "19 Oct 2003"
version    = name ++ ", version " ++ versnum ++ ", " ++ date
copyright  = "Copyright (c) [1995..2003] Manuel M T Chakravarty"
disclaimer = "This software is distributed under the \
             \terms of the GNU Public Licence.\n\
             \NO WARRANTY WHATSOEVER IS PROVIDED. \
             \See the details in the documentation."
