module Version (version, copyright, disclaimer)
where

-- version number is major.minor.patchlvl; don't change the format of the
-- `versnum' line as it is `grep'ed for by a Makefile
--
idstr      = "$Id: Version.hs,v 1.1 2012/05/27 16:49:07 dmwit Exp $"
name       = "C->Haskell Compiler"
versnum    = "0.13.13 (gtk2hs branch)"
versnick   = "\"Bin IO\""
date       = "27 May 2012"
version    = name ++ ", version " ++ versnum ++ " " ++ versnick ++ ", " ++ date
copyright  = "Copyright (c) [1999..2004] Manuel M T Chakravarty"
disclaimer = "This software is distributed under the \
             \terms of the GNU Public Licence.\n\
             \NO WARRANTY WHATSOEVER IS PROVIDED. \
             \See the details in the documentation."
